@@ -34,12 +34,12 @@
 #include <Phonon/MediaObject>
 #include <Phonon/AudioOutput>
 
-MainWindowView::MainWindowView(QWidget *parent) : m_ui(new Ui::MainWindowView)
+MainWindowView::MainWindowView(MainWindow *parent)
+    : m_ui(new Ui::MainWindowView)
+    , m_parent(parent)
+    , m_model(0)
+    , m_pt(0)
 {
-	m_parent = static_cast<MainWindow*> (parent);
-	m_model = 0;
-	m_pt = 0;
-	m_selectedItem = 0;
     m_ui->setupUi(this);
 	m_ui->splitter->setStretchFactor(1, 3);
 	connect(m_ui->fileKurl, SIGNAL(urlSelected(KUrl)), this, SLOT(fileSelected()));
@@ -71,10 +71,10 @@ MainWindowView::~MainWindowView()
 
 void MainWindowView::playSound()
 {
-
-    if(!m_selectedItem)
+    if(!m_ui->treeView->selectionModel()->hasSelection())
          return;
-    int type = m_selectedItem->data(ThemeModel::CardTypeRole).toInt();
+    QModelIndex selectedIdx = m_ui->treeView->selectionModel()->selectedIndexes().first();
+    int type = selectedIdx.data(ThemeModel::CardTypeRole).toInt();
     if(type != CARD_SOUND && type != CARD_SOUNDLOGIC && type != CARD_FOUND)
          return;
     Phonon::MediaSource media(m_ui->fileKurl->startDir().path() + '/' + m_ui->fileKurl->text());
@@ -88,9 +88,16 @@ void MainWindowView::playSound()
 
 void MainWindowView::setModel(ThemeModel *model)
 {
+    if(m_model)
+        disconnect(m_model, 0, this, 0);
 	m_model = model;
-	m_ui->treeView->setModel(model);
-    connect(m_ui->treeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(selectionChanged(QItemSelection,QItemSelection)));
+    if(m_model) {
+        connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SIGNAL(changed()));
+        connect(m_model, SIGNAL(rowsRemoved(QModelIndex,int,int)), SIGNAL(changed()));
+        connect(m_model, SIGNAL(rowsInserted(QModelIndex,int,int)), SIGNAL(changed()));
+        m_ui->treeView->setModel(model);
+        connect(m_ui->treeView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), SLOT(selectionChanged(QItemSelection,QItemSelection)));
+    }
 }
 
 void MainWindowView::widgetsHide()
@@ -200,12 +207,12 @@ bool MainWindowView::check()
 
 void MainWindowView::addFeature(int index)
 {
-    if(!m_model || m_model->rowCount() == 0 || !m_selectedItem || index == -1)
+    if(!m_model || m_model->rowCount() == 0 || !m_ui->treeView->selectionModel()->hasSelection() || index == -1)
         return;
-    QStandardItem *paren = m_selectedItem;
+    QModelIndex paren = m_ui->treeView->selectionModel()->selectedIndexes().first();
     m_ui->moreButton->setCurrentIndex(-1);
-    if(m_selectedItem->data(ThemeModel::CardTypeRole).toInt())
-        paren = m_selectedItem->parent();
+    if(paren.parent().isValid())
+        paren = paren.parent();
     qDebug() << "addFeature called" << index;
     CardType newType;
     switch(index)
@@ -228,38 +235,32 @@ void MainWindowView::addFeature(int index)
             break;
     }
     FeatureItem *fi = new FeatureItem(newType, "any", "");
-    m_model->insertFeature(fi, paren);
-    emit changed();
+    m_model->insertFeature(fi, m_model->itemFromIndex(paren));
 
+    m_ui->treeView->expand(fi->parent()->index());
+    m_ui->treeView->selectionModel()->select(fi->index(), QItemSelectionModel::ClearAndSelect);
 }
 
 void MainWindowView::addElement()
 {
-
-    if(!m_model || m_model->rowCount() == 0 || !m_selectedItem || m_selectedItem->data(ThemeModel::CardTypeRole).toInt())
+    if(!m_model)
         return;
-    const ThemeElement el;
-    ElementItem *newItem = new ElementItem (el);
-    QString name = i18n("Element %1", m_model->rowCount()+1);
-    newItem->setText(name);
-    m_model->insertItem(newItem);
-    emit changed();
 
+    ElementItem* item = new ElementItem(i18n("Element %1", m_model->rowCount()+1), ThemeElement());
+    m_model->appendRow(item);
+    m_ui->treeView->selectionModel()->select(item->index(), QItemSelectionModel::ClearAndSelect);
 }
 
 void MainWindowView::deleteElement()
 {
-	if(!m_model || m_model->rowCount() == 0 || !m_selectedItem)
-		return;
-	int new_row = m_selectedItem->row();
-    m_model->removeItem(m_selectedItem);
-    m_selectedItem = 0;
-    emit changed();
-    if(new_row >= m_model->rowCount())
-    	new_row = m_model->rowCount() - 1;
-    QModelIndex index = m_model->index(new_row,0);
-    m_ui->treeView->selectionModel()->select(index, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
+    if(!m_ui->treeView->selectionModel()->hasSelection())
+        return;
+    QModelIndex oldIdx = m_ui->treeView->selectionModel()->selectedIndexes().first();
+    m_model->removeRow(oldIdx.row(), oldIdx.parent());
 
+    int new_row = qMin(oldIdx.row(), m_model->rowCount() - 1);
+    QModelIndex index = m_model->index(new_row, 0);
+    m_ui->treeView->selectionModel()->select(index, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
 }
 void MainWindowView::selectionChanged(const QItemSelection& selected, const QItemSelection&  )
 {
@@ -269,7 +270,6 @@ void MainWindowView::selectionChanged(const QItemSelection& selected, const QIte
 
 void MainWindowView::elementSelected(const QModelIndex & idx)
 {
-    m_selectedItem = m_model->itemFromIndex(idx);
     int type = idx.data(ThemeModel::CardTypeRole).toInt();
     if(!type)
     {
@@ -355,23 +355,24 @@ void MainWindowView::elementSelected(const QModelIndex & idx)
 }
 void MainWindowView::backSelected()
 {
-    QPixmap image;
     QString newFile = m_parent->copyFile(m_ui->backKurl);
     m_ui->backKurl->setText(m_ui->backKurl->url().fileName());
-    image.load(newFile);
+    QPixmap image(newFile);
   	m_ui->pixLabel->setPixmap(scaleImage(image, 100));
     emit changed();
 }
 void MainWindowView::fileSelected()
 {
-    QPixmap image;
     QString newFile = m_parent->copyFile(m_ui->fileKurl);
     m_ui->fileKurl->setText(m_ui->fileKurl->url().fileName());
-    image.load(newFile);
+    QPixmap image(newFile);
   	m_ui->itemLabel->setPixmap(scaleImage(image, 100));
-    m_selectedItem->setData(m_ui->fileKurl->text(),ThemeModel::PathRole);
-    m_selectedItem->setText(m_ui->fileKurl->text());
-    emit changed();
+    
+    Q_ASSERT(m_ui->treeView->selectionModel()->hasSelection());
+    QModelIndex idx = m_ui->treeView->selectionModel()->selectedIndexes().first();
+    QStandardItem* selectedItem = m_model->itemFromIndex(idx);
+    selectedItem->setData(m_ui->fileKurl->text(),ThemeModel::PathRole);
+    selectedItem->setText(m_ui->fileKurl->text());
 }
 
 QPixmap MainWindowView::scaleImage(const QPixmap &i, int max) const
@@ -383,8 +384,14 @@ QPixmap MainWindowView::scaleImage(const QPixmap &i, int max) const
 
 void MainWindowView::wordChanged(const QString &word)
 {
-    m_selectedItem->setData(word, ThemeModel::PathRole);
-    m_selectedItem->setText(word);
-    emit changed();
+    Q_ASSERT(m_ui->treeView->selectionModel()->hasSelection());
+    QModelIndex idx = m_ui->treeView->selectionModel()->selectedIndexes().first();
+    QStandardItem* selectedItem = m_model->itemFromIndex(idx);
+    selectedItem->setData(word, ThemeModel::PathRole);
+    selectedItem->setText(word);
 }
 
+QString MainWindowView::title() const
+{
+    return m_ui->titleEdit->text();
+}
